@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: vm_stats.c,v 1.11 2004/04/07 21:47:50 tdb Exp $
+ * $Id: vm_stats.c,v 1.12 2004/11/06 18:58:23 tdb Exp $
  */
 
 #include <stdio.h>
@@ -25,7 +25,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-int main(int argc, char **argv){
+int main(){
 
 	sg_mem_stats *mem_stats;
 	sg_swap_stats *swap_stats;
