@@ -17,7 +17,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  *
- * $Id: cpu_stats.c,v 1.26 2005/09/24 13:29:22 tdb Exp $
+ * $Id: cpu_stats.c,v 1.27 2006/10/09 13:52:06 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -125,10 +125,10 @@ sg_cpu_stats *sg_get_cpu_stats(){
 			continue;
 		}
 		cpu_now.user+=(long long)cs.cpu_sysinfo.cpu[CPU_USER];
-		cpu_now.iowait+=(long long)cs.cpu_sysinfo.cpu[CPU_WAIT];
 		cpu_now.kernel+=(long long)cs.cpu_sysinfo.cpu[CPU_KERNEL];
 		cpu_now.idle+=(long long)cs.cpu_sysinfo.cpu[CPU_IDLE];
-		cpu_now.swap+=(long long)cs.cpu_sysinfo.cpu[CPU_STATES];
+		cpu_now.iowait+=(long long)cs.cpu_sysinfo.wait[W_IO]+(long long)cs.cpu_sysinfo.wait[W_PIO];
+		cpu_now.swap+=(long long)cs.cpu_sysinfo.wait[W_SWAP];
 	}
 
 	cpu_now.total=cpu_now.user+cpu_now.iowait+cpu_now.kernel+cpu_now.idle+cpu_now.swap;
