@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: network_stats.c,v 1.77 2006/01/22 18:10:39 tdb Exp $
+ * $Id: network_stats.c,v 1.80 2006/10/09 17:25:06 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -230,10 +230,18 @@ sg_network_io_stats *sg_get_network_io_stats(int *entries){
 
 			/* Create new network_stats */
 			if (VECTOR_RESIZE(network_stats, interfaces + 1) < 0) {
+				kstat_close(kc);
 				return NULL;
 			}
 			network_stat_ptr=network_stats+interfaces;
 
+			/* Read interface name */
+			if (sg_update_string(&network_stat_ptr->interface_name,
+					     ksp->ks_name) < 0) {
+				kstat_close(kc);
+				return NULL;
+			}
+
 			/* Finish reading rx */
 			network_stat_ptr->rx=knp->VALTYPE;
 
@@ -273,11 +281,6 @@ sg_network_io_stats *sg_get_network_io_stats(int *entries){
 			}
 			network_stat_ptr->collisions=knp->value.ui32;
 
-			/* Read interface name */
-			if (sg_update_string(&network_stat_ptr->interface_name,
-					     ksp->ks_name) < 0) {
-				return NULL;
-			}
 
 			/* Store systime */
 			network_stat_ptr->systime=time(NULL);
@@ -665,6 +668,7 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 
 	if ((sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0) {
 		sg_set_error_with_errno(SG_ERROR_SOCKET, NULL);
+		kstat_close(kc);
 		return NULL;
 	}
 
@@ -681,6 +685,7 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 			}
 
 			if (VECTOR_RESIZE(network_iface_stats, ifaces + 1) < 0) {
+				kstat_close(kc);
 				return NULL;
 			}
 			network_iface_stat_ptr = network_iface_stats + ifaces;
@@ -688,11 +693,24 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 
 			if (sg_update_string(&network_iface_stat_ptr->interface_name,
 					     ksp->ks_name) < 0) {
+				kstat_close(kc);
 				return NULL;
 			}
 
 			if ((ifr.ifr_flags & IFF_UP) != 0) {
-				network_iface_stat_ptr->up = 1;
+				if ((knp = kstat_data_lookup(ksp, "link_up")) != NULL) {
+					/* take in to account if link
+					 * is up as well as interface */
+					if (knp->value.ui32 != 0u) {
+						network_iface_stat_ptr->up = 1;
+					} else {
+						network_iface_stat_ptr->up = 0;
+					}
+				}
+				else {
+					/* maintain compatibility */
+					network_iface_stat_ptr->up = 1;
+				}
 			} else {
 				network_iface_stat_ptr->up = 0;
 			}
