@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: swap_stats.c,v 1.24 2005/09/24 13:29:23 tdb Exp $
+ * $Id: swap_stats.c,v 1.25 2007/07/05 16:46:06 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -187,7 +187,7 @@ sg_swap_stats *sg_get_swap_stats(){
 	for (n = 0; ; ++n) {
 		mib[mibsize] = n;
 		size = sizeof xsw;
-		if (sysctl(mib, mibsize + 1, &xsw, &size, NULL, NULL) < 0) {
+		if (sysctl(mib, mibsize + 1, &xsw, &size, NULL, 0) < 0) {
 			break;
 		}
 		if (xsw.xsw_version != XSWDEV_VERSION) {
